/*
 * This file is part of the Owl Positioning System (OwlPS) project.
 * It is subject to the copyright notice and license terms in the
 * COPYRIGHT.t2t file found in the top-level directory of this
 * distribution and at
 *   http://code.lm7.fr/p/owlps/source/tree/master/COPYRIGHT.t2t
 * No part of the OwlPS Project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms
 * contained in the COPYRIGHT.t2t file; the COPYRIGHT.t2t file must be
 * distributed along with this file, either separately or by replacing
 * this notice by the COPYRITGHT.t2t file's contents.
 */


#include "fbcm.hh"
#include "stock.hh"
#include "configuration.hh"



Result FBCM::compute(const Request &_request)
{
  // If the autocalibration is activated, we have to regenerate the
  // Friis indexes each time we calculate a position
  if (Configuration::autocalibration_enabled())
    Stock::update_all_friis_indexes() ;
  return TrilaterationAlgorithm::compute(_request) ;
}


float FBCM::estimate_distance(const Measurement &measurement)
{
  double constant_term = make_constant_term(measurement) ;
  const float &average_dbm = measurement.get_average_dbm() ;
  const CapturePoint *cp = measurement.get_cp() ;
  return pow(10, (constant_term - average_dbm) /
             (10 * friis_index(cp))) ;
}


inline float FBCM::friis_index(const CapturePoint *const cp) const
{
  return cp->get_friis_index() ;
}
