/*
 * This file is part of the Owl Positioning System (OwlPS) project.
 * It is subject to the copyright notice and license terms in the
 * COPYRIGHT.t2t file found in the top-level directory of this
 * distribution and at
 *   http://code.lm7.fr/p/owlps/source/tree/master/COPYRIGHT.t2t
 * No part of the OwlPS Project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms
 * contained in the COPYRIGHT.t2t file; the COPYRIGHT.t2t file must be
 * distributed along with this file, either separately or by replacing
 * this notice by the COPYRITGHT.t2t file's contents.
 */


#ifndef _OWLPS_POSITIONING_OUTPUTTCPSOCKETEVAAL_HH_
#define _OWLPS_POSITIONING_OUTPUTTCPSOCKETEVAAL_HH_

class Point3D ;

#include "outputnetworksocket.hh"

/// Sends results to a remote host by TCP (EvAAL competition format)
/**
 * The results are sent through an TCP socket as a string value,
 * conforming to the EvAAL competition format.
 */
class OutputTCPSocketEvAAL: public OutputNetworkSocket
{
protected:
  /** @name Operations */
  //@{
  /// Initialises the socket
  bool init_socket(void) ;
  /// Sends a string through the socket
  bool send_data(const std::string &data) const ;
  /// Receives and parse the acknowledgement from the server
  bool acknowledge(void) const ;
  //@}

public:
  OutputTCPSocketEvAAL(const std::string &_remote_host,
                       const uint_fast16_t _remote_port) ;

  /** @name Operations */
  //@{
  /// Sends a list of results through the socket
  void write(const ResultList &results) ;
  /// Sends a single result through the socket
  void write(const Result &result) ;
  /// Get the area of interest number from the position
  int area_of_interest_number(const Point3D &position) const ;
  //@}
} ;



#endif // _OWLPS_POSITIONING_OUTPUTTCPSOCKETEVAAL_HH_
